/*
 *                            *
 *                           * *
 *                          *   *
 *                     ***************
 *                      * *       * *
 *                       *  MUMPS  *
 *                      * *       * *
 *                     ***************
 *                          *   *
 *                           * *
 *                            *
 *
 *   fmadm.c
 *    FreeM Administrative Tool
 *
 *  
 *   Author: John P. Willis <jpw@coherent-logic.com>
 *    Copyright (C) 1998 MUG Deutschland
 *    Copyright (C) 2020 Coherent Logic Development LLC
 *
 *   Last modified: 14 Sep 2020
 * 
 **/

#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "fmadm.h"
#include "errmsg.h"
#include "iniconf.h"
#include "init.h"

/* namespace configuration */
char fma_namespace[STRLEN];
char fma_locktab[STRLEN];
char fma_zallotab[STRLEN];
char fma_routine_path[STRLEN];
char fma_global_path[STRLEN];
char fma_pct_global_path[STRLEN];
char fma_pct_routine_path[STRLEN];

/* miscellaneous global state */
char obj_str[STRLEN];

extern char config_file[4096];

int main (int argc, char **argv)
{
    char action[STRLEN];    
    char tmp[STRLEN];

    short act = -1;
    short obj = -1;

    char **opts;
    int optc = argc - 3;
    int result;

    int i = 0;
    int j = 1;

    /* need at least command, action, object, namespace */
    if (argc < 4) return fmadm_usage();

    /* allocate opts array */
    
    /* first dimension */
    if ((opts = (char *) malloc (FMA_MAXARGS * sizeof (char *))) == NULL) {
        fprintf (stderr, "fmadm [FATAL]:  could not acquire memory\n");
        return 1;
    } 

    /* second dimension */
    for (i = 0; i < FMA_MAXARGS; i++) {
        if ((opts[i] = (char *) malloc (STRLEN * sizeof (char *))) == NULL) {
            fprintf (stderr, "fmadm [FATAL]:  could not acquire memory\n");
            return 1;
        } 
    }

    /* copy argv[4] through argv[argc - 1] to opts[1] through opts[argc - 3] */
    
    strncpy (opts[0], argv[0], STRLEN - 1); /* preserve argv[0] */

    for (i = 4; i < argc; i++) {
        if (i > FMA_MAXARGS) return fmadm_usage(); /* bail if we're going to overrun the array */
        strncpy (opts[j++], argv[i], STRLEN - 1);
    }

    strncpy (action, argv[1], STRLEN - 1);
    strncpy (obj_str, argv[2], STRLEN - 1);
    strncpy (fma_namespace, argv[3], STRLEN - 1);
    strncpy (config_file, "/etc/freem.conf", 4096);

    if (strncmp (action, "list", STRLEN - 1) == 0) act = ACT_LIST;
    else if (strncmp (action, "examine", STRLEN - 1) == 0) act = ACT_EXAMINE;
    else if (strncmp (action, "verify", STRLEN - 1) == 0) act = ACT_VERIFY;
    else if (strncmp (action, "compact", STRLEN - 1) == 0) act = ACT_COMPACT;
    else if (strncmp (action, "repair", STRLEN - 1) == 0) act = ACT_REPAIR;
    else if (strncmp (action, "create", STRLEN - 1) == 0) act = ACT_CREATE;
    else if (strncmp (action, "remove", STRLEN - 1) == 0) act = ACT_REMOVE;
    else if (strncmp (action, "import", STRLEN - 1) == 0) act = ACT_IMPORT;
    else if (strncmp (action, "export", STRLEN - 1) == 0) act = ACT_EXPORT;
    else if (strncmp (action, "backup", STRLEN - 1) == 0) act = ACT_BACKUP;
    else if (strncmp (action, "restore", STRLEN - 1) == 0) act = ACT_RESTORE;
    else if (strncmp (action, "migrate", STRLEN - 1) == 0) act = ACT_MIGRATE;
    else if (strncmp (action, "edit", STRLEN -1) == 0) act = ACT_EDIT;
    else return fmadm_usage();

    if (strncmp (obj_str, "lock", STRLEN - 1) == 0) obj = OBJ_LOCK;
    else if (strncmp (obj_str, "zallocate", STRLEN - 1) == 0) obj = OBJ_ZALLOC;
    else if (strncmp (obj_str, "journal", STRLEN - 1) == 0) obj = OBJ_JOURNAL;
    else if (strncmp (obj_str, "namespace", STRLEN - 1) == 0) obj = OBJ_NAMESPACE;
    else if (strncmp (obj_str, "global", STRLEN - 1) == 0) obj = OBJ_GLOBAL;
    else if (strncmp (obj_str, "routine", STRLEN - 1) == 0) obj = OBJ_ROUTINE;
    else if (strncmp (obj_str, "job", STRLEN - 1) == 0) obj = OBJ_JOB;
    else return fmadm_usage();

    if (get_conf (fma_namespace, "locktable_file", fma_locktab) == FALSE) {
        fprintf (stderr, "fmadm:  cannot determine LOCK table location for namespace %s\n", fma_namespace);
        return 1;
    }

    if (get_conf (fma_namespace, "zalloctab_file", fma_zallotab) == FALSE) {
        fprintf (stderr, "fmadm:  cannot determine ZALLOCATE table location for namespace %s\n", fma_namespace);
        return 1;
    }   

    if (get_conf (fma_namespace, "routines_path", fma_routine_path) == FALSE) {
        fprintf (stderr, "fmadm:  cannot determine routine path for namespace %s\n", fma_namespace);
        return 1;
    }   

    if (get_conf (fma_namespace, "globals_path", fma_global_path) == FALSE) {
        fprintf (stderr, "fmadm:  cannot determine global path for namespace %s\n", fma_namespace);
        return 1;
    }   

    if (get_conf ("SYSTEM", "globals_path", fma_pct_global_path) == FALSE) {
        fprintf (stderr, "fmadm:  cannot determine %% global path for namespace %s\n", "SYSTEM");
        return 1;
    }

    if (get_conf ("SYSTEM", "routines_path", fma_pct_routine_path) == FALSE) {
        fprintf (stderr, "fmadm:  cannot determine %% routine path for namespace %s\n", "SYSTEM");
        return 1;
    }


    strcpy (gloplib, fma_pct_global_path);
    stcnv_c2m (gloplib);

    strcpy (glopath, fma_global_path);
    stcnv_c2m (glopath);


    switch (act) {
        case ACT_LIST: return fm_list (obj, optc, opts);
        case ACT_EXAMINE: return fm_examine (obj, optc, opts);
        case ACT_VERIFY: return fm_verify (obj, optc, opts);
        case ACT_COMPACT: return fm_compact (obj, optc, opts);
        case ACT_REPAIR: return fm_repair (obj, optc, opts);
        case ACT_CREATE: return fm_create (obj, optc, opts);
        case ACT_REMOVE: return fm_remove (obj, optc, opts);
        case ACT_IMPORT: return fm_import (obj, optc, opts);
        case ACT_EXPORT: return fm_export (obj, optc, opts);
        case ACT_BACKUP: return fm_backup (obj, optc, opts);
        case ACT_RESTORE: return fm_restore (obj, optc, opts);
        case ACT_MIGRATE: return fm_migrate (obj, optc, opts);
        case ACT_EDIT: return fm_edit (obj, optc, opts);
        default: return fmadm_usage();
    }

    return 0;   /* should never be reached */

} /* main() */

int fmadm_usage (void)
{

    fprintf (stderr, "\nusage:  fmadm <action> <object> <namespace> [OPTIONS]\n\n");

    fprintf (stderr, "        <action> can be one of:\n");
    fprintf (stderr, "            list, examine, verify, compact, repair, create, remove,\n");
    fprintf (stderr, "            import, export, backup, restore, migrate, edit\n\n");

    fprintf (stderr, "        <object> can be one of:\n");
    fprintf (stderr, "            lock, zallocate, journal, namespace, global, routine, job\n\n");

    fprintf (stderr, "    Not all actions are valid for all objects. Please see the FreeM manual\n");
    fprintf (stderr, "    for details on fmadm usage and options.\n\n");

    return 1;

} /* fmadm_usage() */

int fm_list (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_LOCK:
            return fma_locks_list (optc, options);

        case OBJ_ROUTINE:
            return fma_routines_list (optc, options);

        case OBJ_GLOBAL:
            return fma_globals_list (optc, options);

        case OBJ_JOB:
            return fma_jobs_list (optc, options);

        default:
            fprintf (stderr, "fmadm:  'list' is an invalid action for '%s'\n", obj_str);
            return 1;

    }


} /* fm_list() */

int fm_examine (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_ROUTINE:
            return fma_routines_examine (optc, options);

        case OBJ_GLOBAL:
            return fma_globals_examine (optc, options);

        case OBJ_JOB:
            return fma_jobs_examine (optc, options);

        default:
            fprintf (stderr, "fmadm:  'examine' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_examine() */

int fm_verify (short object, int optc, char **options)
{

    switch (object) {

        default:
            fprintf (stderr, "fmadm:  'examine' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_verify() */ 

int fm_compact (short object, int optc, char **options)
{

    switch (object) {

        default:
            fprintf (stderr, "fmadm:  'compact' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_compact() */

int fm_repair (short object, int optc, char **options)
{

    switch (object) {

        default:
            fprintf (stderr, "fmadm:  'repair' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_repair() */

int fm_create (short object, int optc, char **options)
{

    switch (object) {

        default:
            fprintf (stderr, "fmadm:  'create' is an invalid action for '%s'\n", obj_str);
            return 1;

    }
} /* fm_create() */

int fm_remove (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_LOCK:
            return fma_locks_remove (optc, options);

        case OBJ_ROUTINE:
            return fma_routines_remove (optc, options);

        case OBJ_GLOBAL:
            return fma_globals_remove (optc, options);

        default:
            fprintf (stderr, "fmadm:  'remove' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_remove() */

int fm_import (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_ROUTINE: 
            return fma_routines_import (optc, options);

        default:
            fprintf (stderr, "fmadm:  'import' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_import() */

int fm_export (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_ROUTINE: 
            return fma_routines_export (optc, options);

        default:
            fprintf (stderr, "fmadm:  'export' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_export() */

int fm_backup (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_ROUTINE: 
            return fma_routines_backup (optc, options);

        default:
            fprintf (stderr, "fmadm:  'backup' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_backup() */

int fm_restore (short object, int optc, char **options)
{

    switch (object) {

        default:
            fprintf (stderr, "fmadm:  'restore' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_restore() */

int fm_migrate (short object, int optc, char **options)
{

    switch (object) {

        default:
            fprintf (stderr, "fmadm:  'migrate' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_migrate() */

int fm_edit (short object, int optc, char **options)
{

    switch (object) {

        case OBJ_ROUTINE:
            return fma_routines_edit (optc, options);

        default:
            fprintf (stderr, "fmadm:  'edit' is an invalid action for '%s'\n", obj_str);
            return 1;

    }

} /* fm_edit() */



