/*
 *                            *
 *                           * *
 *                          *   *
 *                     ***************
 *                      * *       * *
 *                       *  MUMPS  *
 *                      * *       * *
 *                     ***************
 *                          *   *
 *                           * *
 *                            *
 *
 *   jnldump.c
 *    Utility for dumping the contents of FreeM journals
 *
 *  
 *   Author: John P. Willis <jpw@coherent-logic.com>
 *    Copyright (C) 1998 MUG Deutschland
 *    Copyright (C) 2020 Coherent Logic Development LLC
 * 
 **/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>

#include <unistd.h>
#include <errno.h>

#include "jnldefs.h"

void cvt_key (char *buf, char *key);

int main(int argc, char **argv, char **envp)
{
	register int i;
	int fd;
	unsigned long siz;
	unsigned long ct;

	char buf[2048];
	char kbuf[1024];

	jnl_hdr_t hdr;
	jnl_ent_t ent;

	if (argc != 2) {
		fprintf (stderr, "usage:  jnldump <path-to-journal-file>\n");

		exit (1);
	}

	if ((fd = open (argv[1], O_RDONLY)) == -1) {
		fprintf (stderr, "error: error %d opening journal file %s\n", errno, argv[1]);

		exit (2);
	}

	if (read (fd, &hdr, sizeof (jnl_hdr_t)) == -1) {
		fprintf (stderr, "error: error %d reading header from journal file %s\n", errno, argv[1]);

		exit (3);
	}

	printf ("Journal Format Version:\t%d\n", hdr.fmt_version);

	lseek (fd, 0L, SEEK_SET);
	siz = lseek (fd, 0L, SEEK_END);

	ct = (siz - sizeof (jnl_hdr_t)) / sizeof (jnl_ent_t);

	printf ("Journal Entries:\t%d\n", ct);

	lseek (fd, sizeof (jnl_hdr_t), SEEK_SET);

	for (i = 0; i < ct; i++) {

		read (fd, &ent, sizeof (jnl_ent_t));

		switch (ent.action) {
			
			case JNLA_TSTART:
				strcpy (buf, "TSTART");
				break;

			case JNLA_TROLLBACK:
				strcpy (buf, "TROLLBACK");
				break;

			case JNLA_TCOMMIT:
				strcpy (buf, "TCOMMIT");
				break;

			case JNLA_SET:
				strcpy (buf, "SET");
				break;

			case JNLA_KILL:
				strcpy (buf, "KILL");
				break;
			
		}

		cvt_key (kbuf, ent.key);

		printf ("%s\t\ttransaction id = %s:%ld pid = %ld action = %s glvn = %s\n", ctime (&ent.ts), ent.host_id, ent.tran_id, ent.pid, buf, kbuf);

	}

	exit (0);
}

void cvt_key (char *buf, char *key)
{
	int i;
	int j = 1;
	int first = 1;

	if (key[0] == '\0') {
		buf[0] = '\0';

		return;
	}

	buf[0] = '^';

	for(i = 0; i < 1023; i++) {
		
		switch (key[i]) {

			case '\201':
				if (first == 0) {
					buf[j] = ')';
					buf[j + 1] = '\0';
				}
				else {
					buf[j] = '\0';
				}
				return;

			case '\202':
				if (first == 1) {
					buf[j] = '(';
					first = 0;
				}
				else {
					buf[j] = ',';
				}

				break;

			default:
				buf[j] = key[i];
				break;
		}

		j++;
	}

}